%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program computes the efficient GMM estimator
% for the random coefficients model
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all;
clc;

% Sample size 
N=100;
T=2;

% True parameter
theta0=1;

% Size of the discretization 
Ny=1000;
Na=100;

% Quasi-random draws 
qA = qrandstream('halton',1,'Skip',1e3,'Leap',1e2);
qY = qrandstream('halton',T,'Skip',1e3,'Leap',1e2);

UA = qrand(qA,Na);
UY = qrand(qY,Ny);
UY=kron(UY(:,1),[0;1])+kron(UY(:,2),[1;0]);

UALPH=rand(Ny,1);


% Number of singular values 
ks=10;

% Simulated Y's (true distribution)
thetavec=kron(ones(Ny,1),[1;theta0]);

alpha0=norminv(UALPH);
alpha=kron(alpha0,ones(T,1))+1;

Ys=thetavec.*alpha+norminv(UY);

% pi_alpha (truth)
A=1+norminv(UA);

% Compute Ls(theta0)
thetavec=kron(ones(Ny,1),[1;theta0]);
L_t=normpdf(Ys*ones(1,Na)-thetavec*A');
L0s=zeros(Ny,Na);
for j=1:Ny
    L0s(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end

% PiYs (=1/f_ys)
PiYs=Na./sum(L0s,2);


% Compute dLs1 
dL_t=(kron(ones(Ny,1),[0;1])*A').*(Ys*ones(1,Na)-thetavec*A')...
    .*normpdf(Ys*ones(1,Na)-thetavec*A');
dLs1=zeros(Ny,Na);
for j=1:Ny
    dLs1(j,:)=sum(dL_t((j-1)*T+1:j*T,:)./L_t((j-1)*T+1:j*T,:)).*L0s(j,:);
end

Hs_opt=PiYs.*sum(dLs1,2)/Na;


% Loop on theta
vectheta=(.5:.01:1.5)';
[nsig csig]=size(vectheta);

Matcoeff_opt=zeros(Na,nsig);



for j1=1:nsig;
theta=vectheta(j1);
thetavec=kron(ones(Ny,1),[1;theta]);

% Compute Ls 
L_t=normpdf(Ys*ones(1,Na)-thetavec*A');
Ls=zeros(Ny,Na);
for j=1:Ny
    Ls(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end

% Left-multiply by PiYs
Ls=(PiYs*ones(1,Na)).*Ls;

% Singular value decomposition
[U S V]=svd(Ls,0);

% Moore-Penrose inverse (ks singular values)
pinvLs=V(:,1:ks)*inv(S(1:ks,1:ks))*U(:,1:ks)';

% Compute least squares coefficients
Matcoeff_opt(:,j1)=pinvLs*Hs_opt;

end



% Number of Monte Carlo simulations @
Nmc=1000;

% Matrices where results are stored 
Totalresopt=zeros(Nmc,1);
TotalresRE=zeros(Nmc,1);

for jmc=1:Nmc

% Y (data)
thetavec=kron(ones(N,1),[1;theta0]);
alpha0=randn(N,1);
alpha=kron(alpha0,ones(T,1))+1;

Y=thetavec.*alpha+randn(N*T,1);

% Compute L(theta0)
L_t=normpdf(Y*ones(1,Na)-thetavec*A');
L0=zeros(N,Na);
for j=1:N
    L0(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end

% PiY (=1/f_ys)
PiY=Na./sum(L0,2);

% Compute dL(theta0) 
dL_t=(kron(ones(N,1),[0;1])*A').*(Y*ones(1,Na)-thetavec*A')...
    .*normpdf(Y*ones(1,Na)-thetavec*A');
dL0=zeros(N,Na);
for j=1:N
    dL0(j,:)=sum(dL_t((j-1)*T+1:j*T,:)./L_t((j-1)*T+1:j*T,:)).*L0(j,:);
end

H_opt=PiY.*sum(dL0,2)/Na;

% Grid on theta
resultsopt=zeros(nsig,1);
resultsRE=zeros(nsig,1);

% Start loop on theta

for j1=1:nsig;
theta=vectheta(j1);
thetavec=kron(ones(N,1),[1;theta]);

% Compute L 
L_t=normpdf(Y*ones(1,Na)-thetavec*A');
L=zeros(N,Na);
for j=1:N
    L(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end

% Left-multiply by PiY
L=(PiY*ones(1,Na)).*L;

% save results 
resultsopt(j1)=sum(H_opt-L*Matcoeff_opt(:,j1)).^2;

% Infeasible random effects 
L_tRE=normpdf(Y*ones(1,N)-thetavec*(1+alpha0'));
LRE=zeros(N,N);
for j=1:N
    LRE(j,:)=prod(L_tRE((j-1)*T+1:j*T,:));
end
resultsRE(j1)=-sum(log(mean(LRE,2)));



% end loop on parameters 
end


[M I1]=min(resultsopt);
Totalresopt(jmc)=vectheta(I1);

[M IRE]=min(resultsRE);
TotalresRE(jmc)=vectheta(IRE);



jmc

% end loop on simulations
end

% Graph
plot(vectheta,resultsopt)

% efficient GMM
mean(Totalresopt)
std(Totalresopt)

% infeasible random-effects
mean(TotalresRE)
std(TotalresRE)
