%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program computes the efficient GMM estimator
% for the Tobit model
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
clc;

% Sample size 
N=100;
T=2;

% True variance 
sigma20=1;

% c is the censoring point (known) 
c=0;

% Size of the discretization 
Ny=1000;
Na=100;

% Quasi-random draws 
qA = qrandstream('halton',1,'Skip',1e3,'Leap',1e2);
qY = qrandstream('halton',T,'Skip',1e3,'Leap',1e2);

UA = qrand(qA,Na);
UY = qrand(qY,Ny);
UY=kron(UY(:,1),[0;1])+kron(UY(:,2),[1;0]);

UALPH=rand(Ny,1);


% Number of singular values 
ks=10;

% Simulated Y's (true distribution)
alpha0=norminv(UALPH);
alpha=kron(alpha0,ones(T,1));

Ys=alpha+sqrt(sigma20)*norminv(UY);
Ys=(Ys>c).*Ys+(Ys<=c).*c;

% pi_alpha (truth)
A=norminv(UA);

% Compute Ls(theta0)
L_t=((Ys==c)*ones(1,Na)).*normcdf((c*ones(Ny*T,Na)-ones(Ny*T,1)*A')/sqrt(sigma20))+...
    ((Ys>c)*ones(1,Na)).*(1/sqrt(sigma20)).*normpdf((Ys*ones(1,Na)-ones(Ny*T,1)*A')/sqrt(sigma20));
L0s=zeros(Ny,Na);
for j=1:Ny
    L0s(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end

% PiYs (=1/f_ys)
PiYs=Na./sum(L0s,2);


% Compute dLs1 
dL_t=((Ys==c)*ones(1,Na)).*(-1/sqrt(sigma20)^2).*(c*ones(Ny*T,Na)-ones(Ny*T,1)*A').*...
    normpdf((c*ones(Ny*T,Na)-ones(Ny*T,1)*A')/sqrt(sigma20))...
    +((Ys>c)*ones(1,Na)).*(-1/sqrt(sigma20)^2).*normpdf((Ys*ones(1,Na)-ones(Ny*T,1)*A')/sqrt(sigma20))...
    -((Ys>c)*ones(1,Na)).*(-1/sqrt(sigma20)^3).*(Ys*ones(1,Na)-ones(Ny*T,1)*A').*...
    ((Ys*ones(1,Na)-ones(Ny*T,1)*A')/sqrt(sigma20)).*normpdf((Ys*ones(1,Na)-ones(Ny*T,1)*A')/sqrt(sigma20));
dL_t=dL_t/(2*sqrt(sigma20));
dLs1=zeros(Ny,Na);
for j=1:Ny
    dLs1(j,:)=sum(dL_t((j-1)*T+1:j*T,:)./L_t((j-1)*T+1:j*T,:)).*L0s(j,:);
end

Hs_opt=PiYs.*sum(dLs1,2)/Na;


% Loop on sigma^2 
vecsigma=(.5:.01:1.5)';
[nsig csig]=size(vecsigma);

Matcoeff_opt=zeros(Na,nsig);


for j1=1:nsig;
sigma=vecsigma(j1);

% Compute Ls 
L_t=((Ys==c)*ones(1,Na)).*normcdf((c*ones(Ny*T,Na)-ones(Ny*T,1)*A')/sqrt(sigma))+...
    ((Ys>c)*ones(1,Na)).*(1/sqrt(sigma)).*normpdf((Ys*ones(1,Na)-ones(Ny*T,1)*A')/sqrt(sigma));
Ls=zeros(Ny,Na);
for j=1:Ny
    Ls(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end

% Left-multiply by PiYs
Ls=(PiYs*ones(1,Na)).*Ls;

% Singular value decomposition
[U S V]=svd(Ls,0);

% Moore-Penrose inverse (ks singular values)
pinvLs=V(:,1:ks)*inv(S(1:ks,1:ks))*U(:,1:ks)';

% Compute least squares coefficients
Matcoeff_opt(:,j1)=pinvLs*Hs_opt;
end



% Number of Monte Carlo simulations @
Nmc=1000;

% Matrices where results are stored 
Totalresopt=zeros(Nmc,1);
TotalresRE=zeros(Nmc,1);

for jmc=1:Nmc

% Y (data) 
alpha0=randn(N,1);
alpha=kron(alpha0,ones(T,1));

Ystar=alpha+sqrt(sigma20)*randn(N*T,1);
Y=(Ystar>c).*Ystar+(Ystar<=c).*c;

% Compute L(theta0)
L0_t=((Y==c)*ones(1,Na)).*normcdf((c*ones(N*T,Na)-ones(N*T,1)*A')/sqrt(sigma20))+...
    ((Y>c)*ones(1,Na)).*(1/sqrt(sigma20)).*normpdf((Y*ones(1,Na)-ones(N*T,1)*A')/sqrt(sigma20));
L0=zeros(N,Na);
for j=1:N
    L0(j,:)=prod(L0_t((j-1)*T+1:j*T,:));
end

% PiY (=1/f_y)
PiY=Na./sum(L0,2);

% Compute dL 
dL_t=((Y==c)*ones(1,Na)).*(-1/sqrt(sigma20)^2).*(c*ones(N*T,Na)-ones(N*T,1)*A').*...
    normpdf((c*ones(N*T,Na)-ones(N*T,1)*A')/sqrt(sigma20))...
    +((Y>c)*ones(1,Na)).*(-1/sqrt(sigma20)^2).*normpdf((Y*ones(1,Na)-ones(N*T,1)*A')/sqrt(sigma20))...
    -((Y>c)*ones(1,Na)).*(-1/sqrt(sigma20)^3).*(Y*ones(1,Na)-ones(N*T,1)*A').*...
    ((Y*ones(1,Na)-ones(N*T,1)*A')/sqrt(sigma20)).*normpdf((Y*ones(1,Na)-ones(N*T,1)*A')/sqrt(sigma20));
dL_t=dL_t/(2*sqrt(sigma20));
dL0=zeros(N,Na);
for j=1:N
    dL0(j,:)=sum(dL_t((j-1)*T+1:j*T,:)./L0_t((j-1)*T+1:j*T,:)).*L0(j,:);
end

H_opt=PiY.*sum(dL0,2)/Na;


% Grid on sigma 
resultsopt=zeros(nsig,1);
resultsRE=zeros(nsig,1);

% Start loop on sigma 

for j1=1:nsig;
sigma=vecsigma(j1);

% Compute L 
L_t=((Y==c)*ones(1,Na)).*normcdf((c*ones(N*T,Na)-ones(N*T,1)*A')/sqrt(sigma))+...
    ((Y>c)*ones(1,Na)).*(1/sqrt(sigma)).*normpdf((Y*ones(1,Na)-ones(N*T,1)*A')/sqrt(sigma));
L=zeros(N,Na);
for j=1:N
    L(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end


% Left-multiply by PiY
L=(PiY*ones(1,Na)).*L;

% save results 
resultsopt(j1)=sum(H_opt-L*Matcoeff_opt(:,j1)).^2;

% Infeasible random effects 
L_tRE=((Y==c)*ones(1,N)).*normcdf((c*ones(N*T,N)-ones(N*T,1)*alpha0')/sqrt(sigma))...
    +((Y>c)*ones(1,N)).*(1/sqrt(sigma)).*normpdf((Y*ones(1,N)-ones(N*T,1)*alpha0')/sqrt(sigma));
LRE=zeros(N,N);
for j=1:N
    LRE(j,:)=prod(L_tRE((j-1)*T+1:j*T,:));
end
resultsRE(j1)=-sum(log(mean(LRE,2)));



% end loop on parameters 
end


[M I]=min(resultsopt);
Totalresopt(jmc)=vecsigma(I);

[M I]=min(resultsRE);
TotalresRE(jmc)=vecsigma(I);

jmc

% end loop on simulations
end

% Graph
plot(vecsigma,[resultsRE resultsopt])

% efficient GMM
mean(sqrt(Totalresopt))
std(sqrt(Totalresopt))

% infeasible random-effects
mean(sqrt(TotalresRE))
std(sqrt(TotalresRE))
