%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program computes the GMM estimators and the infeasible RE estimator,
% for the Tobit model
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all;
clc;

% Sample size 
N=100;
T=2;

% True variance 
sigma20=1;

% c is the censoring point (known) 
c=0;

% Grid for the moment functions 
Mu1=[kron([0;1;-1],[1;1;1]) kron([1;1;1],[0;1;-1])];
Mu2=[kron([0;1;-1;2;-2],[1;1;1;1;1]) kron([1;1;1;1;1],[0;1;-1;2;-2])];
Mu3=[kron([0;1;-1;2;-2;3;-3],[1;1;1;1;1;1;1]) kron([1;1;1;1;1;1;1],[0;1;-1;2;-2;3;-3])];

% Size of the discretization 
Ny=1000;
Na=100;

% Quasi-random draws 
qA = qrandstream('halton',1,'Skip',1e3,'Leap',1e2);
qY = qrandstream('halton',T,'Skip',1e3,'Leap',1e2);

UA = qrand(qA,Na);
UY = qrand(qY,Ny);
UY=kron(UY(:,1),[0;1])+kron(UY(:,2),[1;0]);

% Number of singular values 
ks=10;

% Simulated Y's 
alphaPM=0;
sigma2PM=3;

Ys=alphaPM+sqrt(sigma2PM)*norminv(UY);
Ys=(Ys>c).*Ys+(Ys<=c).*c;

% H_r evaluated at Ys 

matH1s=zeros(Ny,3^2);
for kg=1:3^2
H1=normpdf(Ys-kron(ones(Ny,1),Mu1(kg,:)'));
for j=1:Ny
    matH1s(j,kg)=prod(H1((j-1)*T+1:j*T,:));
end
end

matH2s=zeros(Ny,5^2);
for kg=1:5^2
H2=normpdf(Ys-kron(ones(Ny,1),Mu2(kg,:)'));
for j=1:Ny
    matH2s(j,kg)=prod(H2((j-1)*T+1:j*T,:));
end
end

matH3s=zeros(Ny,7^2);
for kg=1:7^2
H3=normpdf(Ys-kron(ones(Ny,1),Mu3(kg,:)'));
for j=1:Ny
    matH3s(j,kg)=prod(H3((j-1)*T+1:j*T,:));
end
end

% Support of pi_alpha
A=10*UA-5;

% Loop on sigma^2 
vecsigma=(.5:.01:1.5)';
[nsig csig]=size(vecsigma);

Matcoeff1=zeros(Na,3^2,nsig);
Matcoeff2=zeros(Na,5^2,nsig);
Matcoeff3=zeros(Na,7^2,nsig);


for j1=1:nsig;
sigma=vecsigma(j1);

% Compute Ls 
L_t=((Ys==c)*ones(1,Na)).*normcdf((c*ones(Ny*T,Na)-ones(Ny*T,1)*A')/sqrt(sigma))+...
    ((Ys>c)*ones(1,Na)).*(1/sqrt(sigma)).*normpdf((Ys*ones(1,Na)-ones(Ny*T,1)*A')/sqrt(sigma));
Ls=zeros(Ny,Na);
for j=1:Ny
    Ls(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end

% Singular value decomposition
[U S V]=svd(Ls);

% Moore-Penrose inverse (ks singular values)
pinvLs=V(:,1:ks)*inv(S(1:ks,1:ks))*U(:,1:ks)';

% Compute least squares coefficients
for kg=1:3^2
    Matcoeff1(:,kg,j1)=pinvLs*matH1s(:,kg);
end

for kg=1:5^2
    Matcoeff2(:,kg,j1)=pinvLs*matH2s(:,kg);
end

for kg=1:7^2
    Matcoeff3(:,kg,j1)=pinvLs*matH3s(:,kg);
end

end



% Number of Monte Carlo simulations 
Nmc=1000;

% Matrices where results are stored 
Totalres1=zeros(Nmc,1);
Totalres2=zeros(Nmc,1);
Totalres3=zeros(Nmc,1);
TotalresRE=zeros(Nmc,1);
Totalvar1=zeros(Nmc,1);
Totalvar2=zeros(Nmc,1);
Totalvar3=zeros(Nmc,1);
TotalvarRE=zeros(Nmc,1);

for jmc=1:Nmc

% Y (data)
alpha0=randn(N,1);
alpha=kron(alpha0,ones(T,1));

Ystar=alpha+sqrt(sigma20)*randn(N*T,1);
Y=(Ystar>c).*Ystar+(Ystar<=c).*c;

% PiY 
PiY1=(Y==c).*normcdf((c-alphaPM)/sqrt(sigma2PM))...
        +(Y>c).*1/sqrt(sigma2PM).*normpdf((Y-alphaPM)/sqrt(sigma2PM));
PiY=zeros(N,1);
for j=1:N
    PiY(j,:)=prod(PiY1((j-1)*T+1:j*T));
end
        
% H_r evaluated at Y 

matH1=zeros(N,3^2);
for kg=1:3^2
H1=normpdf(Y-kron(ones(N,1),Mu1(kg,:)'));
for j=1:N
    matH1(j,kg)=prod(H1((j-1)*T+1:j*T,:));
end
end

matH2=zeros(N,5^2);
for kg=1:5^2
H2=normpdf(Y-kron(ones(N,1),Mu2(kg,:)'));
for j=1:N
    matH2(j,kg)=prod(H2((j-1)*T+1:j*T,:));
end
end

matH3=zeros(N,7^2);
for kg=1:7^2
H3=normpdf(Y-kron(ones(N,1),Mu3(kg,:)'));
for j=1:N
    matH3(j,kg)=prod(H3((j-1)*T+1:j*T,:));
end
end


% Grid on sigma 
resultsgrid1=zeros(nsig,1);
resultsgrid2=zeros(nsig,1);
resultsgrid3=zeros(nsig,1);
resultsRE=zeros(nsig,1);

% Start loop on sigma 

for j1=1:nsig;
sigma=vecsigma(j1);

% Compute L 
L_t=((Y==c)*ones(1,Na)).*normcdf((c*ones(N*T,Na)-ones(N*T,1)*A')/sqrt(sigma))+...
    ((Y>c)*ones(1,Na)).*(1/sqrt(sigma)).*normpdf((Y*ones(1,Na)-ones(N*T,1)*A')/sqrt(sigma));
L=zeros(N,Na);
for j=1:N
    L(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end

% save results 
resultsgrid1(j1)=sum(sum((matH1-L*Matcoeff1(:,:,j1)).*(PiY*ones(1,3^2))).^2);
resultsgrid2(j1)=sum(sum((matH2-L*Matcoeff2(:,:,j1)).*(PiY*ones(1,5^2))).^2);
resultsgrid3(j1)=sum(sum((matH3-L*Matcoeff3(:,:,j1)).*(PiY*ones(1,7^2))).^2);

% Infeasible random effects 
L_tRE=((Y==c)*ones(1,N)).*normcdf((c*ones(N*T,N)-ones(N*T,1)*alpha0')/sqrt(sigma))...
    +((Y>c)*ones(1,N)).*(1/sqrt(sigma)).*normpdf((Y*ones(1,N)-ones(N*T,1)*alpha0')/sqrt(sigma));
LRE=zeros(N,N);
for j=1:N
    LRE(j,:)=prod(L_tRE((j-1)*T+1:j*T,:));
end
resultsRE(j1)=-sum(log(mean(LRE,2)));



% end loop on parameters 
end


[M I1]=min(resultsgrid1);
Totalres1(jmc)=vecsigma(I1);

[M I2]=min(resultsgrid2);
Totalres2(jmc)=vecsigma(I2);

[M I3]=min(resultsgrid3);
Totalres3(jmc)=vecsigma(I3);

[M IRE]=min(resultsRE);
TotalresRE(jmc)=vecsigma(IRE);

% Variances

% Jacobian (numerical)
I=I1;
if I<nsig-10
    I_per=I+1;
else 
    I_per=I-1;
end

% Compute L 
sigma=vecsigma(I);
L_t=((Y==c)*ones(1,Na)).*normcdf((c*ones(N*T,Na)-ones(N*T,1)*A')/sqrt(sigma))+...
    ((Y>c)*ones(1,Na)).*(1/sqrt(sigma)).*normpdf((Y*ones(1,Na)-ones(N*T,1)*A')/sqrt(sigma));
L=zeros(N,Na);
for j=1:N
    L(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end
ind=(matH1-L*Matcoeff1(:,:,I)).*(PiY*ones(1,3^2));

% Compute L at a different sigma (for computing the jacobian G numerically)
sigma=vecsigma(I_per);
L_t=((Y==c)*ones(1,Na)).*normcdf((c*ones(N*T,Na)-ones(N*T,1)*A')/sqrt(sigma))+...
    ((Y>c)*ones(1,Na)).*(1/sqrt(sigma)).*normpdf((Y*ones(1,Na)-ones(N*T,1)*A')/sqrt(sigma));
L=zeros(N,Na);
for j=1:N
    L(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end
ind_per=(matH1-L*Matcoeff1(:,:,I_per)).*(PiY*ones(1,3^2));

Mom=sum(ind)';
Mom_per=sum(ind_per)';
G=(Mom_per-Mom)/(vecsigma(I_per)-vecsigma(I));

% Outer product
Sig=ind'*ind;

% Variance
Totalvar1(jmc)=inv(G'*G)*G'*Sig*G*inv(G'*G);

% Jacobian (numerical)
I=I2;
if I<nsig-10
    I_per=I+1;
else 
    I_per=I-1;
end

% Compute L 
sigma=vecsigma(I);
L_t=((Y==c)*ones(1,Na)).*normcdf((c*ones(N*T,Na)-ones(N*T,1)*A')/sqrt(sigma))+...
    ((Y>c)*ones(1,Na)).*(1/sqrt(sigma)).*normpdf((Y*ones(1,Na)-ones(N*T,1)*A')/sqrt(sigma));
L=zeros(N,Na);
for j=1:N
    L(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end
ind=(matH2-L*Matcoeff2(:,:,I)).*(PiY*ones(1,5^2));

% Compute L at a different sigma (for computing the jacobian G numerically)
sigma=vecsigma(I_per);
L_t=((Y==c)*ones(1,Na)).*normcdf((c*ones(N*T,Na)-ones(N*T,1)*A')/sqrt(sigma))+...
    ((Y>c)*ones(1,Na)).*(1/sqrt(sigma)).*normpdf((Y*ones(1,Na)-ones(N*T,1)*A')/sqrt(sigma));
L=zeros(N,Na);
for j=1:N
    L(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end
ind_per=(matH2-L*Matcoeff2(:,:,I_per)).*(PiY*ones(1,5^2));

Mom=sum(ind)';
Mom_per=sum(ind_per)';
G=(Mom_per-Mom)/(vecsigma(I_per)-vecsigma(I));

% Outer product
Sig=ind'*ind;

% Variance
Totalvar2(jmc)=inv(G'*G)*G'*Sig*G*inv(G'*G);

% Jacobian (numerical)
I=I3;
if I<nsig-10
    I_per=I+1;
else 
    I_per=I-1;
end

% Compute L 
sigma=vecsigma(I);
L_t=((Y==c)*ones(1,Na)).*normcdf((c*ones(N*T,Na)-ones(N*T,1)*A')/sqrt(sigma))+...
    ((Y>c)*ones(1,Na)).*(1/sqrt(sigma)).*normpdf((Y*ones(1,Na)-ones(N*T,1)*A')/sqrt(sigma));
L=zeros(N,Na);
for j=1:N
    L(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end
ind=(matH3-L*Matcoeff3(:,:,I)).*(PiY*ones(1,7^2));

% Compute L at a different sigma (for computing the jacobian G numerically)
sigma=vecsigma(I_per);
L_t=((Y==c)*ones(1,Na)).*normcdf((c*ones(N*T,Na)-ones(N*T,1)*A')/sqrt(sigma))+...
    ((Y>c)*ones(1,Na)).*(1/sqrt(sigma)).*normpdf((Y*ones(1,Na)-ones(N*T,1)*A')/sqrt(sigma));
L=zeros(N,Na);
for j=1:N
    L(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end
ind_per=(matH3-L*Matcoeff3(:,:,I_per)).*(PiY*ones(1,7^2));

Mom=sum(ind)';
Mom_per=sum(ind_per)';
G=(Mom_per-Mom)/(vecsigma(I_per)-vecsigma(I));

% Outer product
Sig=ind'*ind;

% Variance
Totalvar3(jmc)=inv(G'*G)*G'*Sig*G*inv(G'*G);

jmc

% end loop on simulations
end

% Graph
plot(vecsigma,resultsgrid2)

% GMM, first set of moment functions
mean(sqrt(Totalres1))
std(sqrt(Totalres1))
% Estimate of asymptotic standard errors
mean(sqrt(Totalvar1))/2

% GMM, second set of moment functions
mean(sqrt(Totalres2))
std(sqrt(Totalres2))
% Estimate of asymptotic standard errors
mean(sqrt(Totalvar2))/2

% GMM, third set of moment functions
mean(sqrt(Totalres3))
std(sqrt(Totalres3))
% Estimate of asymptotic standard errors
mean(sqrt(Totalvar3))/2

% Infeasible random effects
mean(sqrt(TotalresRE))
std(sqrt(TotalresRE))


